@@ -36,7 +36,7 @@
 
 class ScriptDebuggerLocal : public ScriptDebugger {
 	bool profiling;
-	float frame_time, idle_time, physics_time, physics_frame_time;
+	float frame_time, process_time, physics_time, physics_frame_time;
 	uint64_t idle_accum;
 	String target_function;
 	Map<String, String> options;
@@ -58,7 +58,7 @@ public:
 
 	virtual void profiling_start();
 	virtual void profiling_end();
-	virtual void profiling_set_frame_times(float p_frame_time, float p_idle_time, float p_physics_time, float p_physics_frame_time);
+	virtual void profiling_set_frame_times(float p_frame_time, float p_process_time, float p_physics_time, float p_physics_frame_time);
 
 	ScriptDebuggerLocal();
 };
