@@ -76,8 +76,6 @@ class CreateDialog : public ConfirmationDialog {
 	void _confirmed();
 	void _text_changed(const String &p_newtext);
 
-	Ref<Texture> _get_editor_icon(const String &p_type) const;
-
 	void add_type(const String &p_type, HashMap<String, TreeItem *> &p_types, TreeItem *p_root, TreeItem **to_select);
 
 	void select_type(const String &p_type);
@@ -110,4 +108,4 @@ public:
 	CreateDialog();
 };
 
-#endif
+#endif // CREATE_DIALOG_H
