@@ -142,8 +142,16 @@ void EditorRunNative::_run_native(int p_idx, int p_platform) {
 	if (debug_navigation) {
 		flags |= EditorExportPlatform::DEBUG_FLAG_VIEW_NAVIGATION;
 	}
+	if (debug_shader_fallbacks) {
+		flags |= EditorExportPlatform::DEBUG_FLAG_SHADER_FALLBACKS;
+	}
 
-	eep->run(preset, p_idx, flags);
+	eep->clear_messages();
+	Error err = eep->run(preset, p_idx, flags);
+	result_dialog_log->clear();
+	if (eep->fill_log_messages(result_dialog_log, err)) {
+		result_dialog->popup_centered_ratio(0.5);
+	}
 }
 
 void EditorRunNative::resume_run_native() {
@@ -188,13 +196,31 @@ bool EditorRunNative::get_debug_navigation() const {
 	return debug_navigation;
 }
 
+void EditorRunNative::set_debug_shader_fallbacks(bool p_debug) {
+	debug_shader_fallbacks = p_debug;
+}
+
+bool EditorRunNative::get_debug_shader_fallbacks() const {
+	return debug_shader_fallbacks;
+}
+
 EditorRunNative::EditorRunNative() {
+	result_dialog = memnew(AcceptDialog);
+	result_dialog->set_title(TTR("Project Run"));
+	result_dialog_log = memnew(RichTextLabel);
+	result_dialog_log->set_custom_minimum_size(Size2(300, 80) * EDSCALE);
+	result_dialog->add_child(result_dialog_log);
+
+	add_child(result_dialog);
+	result_dialog->hide();
+
 	set_process(true);
 	first = true;
 	deploy_dumb = false;
 	deploy_debug_remote = false;
 	debug_collisions = false;
 	debug_navigation = false;
+	debug_shader_fallbacks = false;
 	resume_idx = 0;
 	resume_platform = 0;
 }
