@@ -31,6 +31,7 @@
 #ifndef THEME_EDITOR_PLUGIN_H
 #define THEME_EDITOR_PLUGIN_H
 
+#include "scene/gui/dialogs.h"
 #include "scene/gui/margin_container.h"
 #include "scene/gui/option_button.h"
 #include "scene/gui/scroll_container.h"
@@ -175,7 +176,11 @@ class ThemeItemEditorDialog : public AcceptDialog {
 
 	TabContainer *tc;
 
-	ItemList *edit_type_list;
+	enum TypesTreeAction {
+		TYPES_TREE_REMOVE_ITEM,
+	};
+
+	Tree *edit_type_list;
 	LineEdit *edit_add_type_value;
 	String edited_item_type;
 
@@ -226,13 +231,15 @@ class ThemeItemEditorDialog : public AcceptDialog {
 
 	void _dialog_about_to_show();
 	void _update_edit_types();
-	void _edited_type_selected(int p_item_idx);
+	void _edited_type_selected();
+	void _edited_type_button_pressed(Object *p_item, int p_column, int p_id);
 
 	void _update_edit_item_tree(String p_item_type);
 	void _item_tree_button_pressed(Object *p_item, int p_column, int p_id);
 
 	void _add_theme_type(const String &p_new_text);
 	void _add_theme_item(Theme::DataType p_data_type, String p_item_name, String p_item_type);
+	void _remove_theme_type(const String &p_theme_type);
 	void _remove_data_type_items(Theme::DataType p_data_type, String p_item_type);
 	void _remove_class_items();
 	void _remove_custom_items();
@@ -256,6 +263,42 @@ public:
 	ThemeItemEditorDialog();
 };
 
+class ThemeTypeDialog : public ConfirmationDialog {
+	GDCLASS(ThemeTypeDialog, ConfirmationDialog);
+
+	Ref<Theme> edited_theme;
+	bool include_own_types = false;
+
+	String pre_submitted_value;
+
+	LineEdit *add_type_filter;
+	ItemList *add_type_options;
+	ConfirmationDialog *add_type_confirmation;
+
+	void _dialog_about_to_show();
+	void ok_pressed();
+
+	void _update_add_type_options(const String &p_filter = "");
+
+	void _add_type_filter_cbk(const String &p_value);
+	void _add_type_options_cbk(int p_index);
+	void _add_type_dialog_entered(const String &p_value);
+	void _add_type_dialog_activated(int p_index);
+
+	void _add_type_selected(const String &p_type_name);
+	void _add_type_confirmed();
+
+protected:
+	void _notification(int p_what);
+	static void _bind_methods();
+
+public:
+	void set_edited_theme(const Ref<Theme> &p_theme);
+	void set_include_own_types(bool p_enable);
+
+	ThemeTypeDialog();
+};
+
 class ThemeTypeEditor : public MarginContainer {
 	GDCLASS(ThemeTypeEditor, MarginContainer);
 
@@ -274,9 +317,6 @@ class ThemeTypeEditor : public MarginContainer {
 
 	OptionButton *theme_type_list;
 	Button *add_type_button;
-	ConfirmationDialog *add_type_dialog;
-	LineEdit *add_type_filter;
-	ItemList *add_type_options;
 
 	CheckButton *show_default_items_button;
 
@@ -287,26 +327,31 @@ class ThemeTypeEditor : public MarginContainer {
 	VBoxContainer *icon_items_list;
 	VBoxContainer *stylebox_items_list;
 
+	LineEdit *type_variation_edit;
+	Button *type_variation_button;
+	Label *type_variation_locked;
+
+	enum TypeDialogMode {
+		ADD_THEME_TYPE,
+		ADD_VARIATION_BASE,
+	};
+
+	TypeDialogMode add_type_mode = ADD_THEME_TYPE;
+	ThemeTypeDialog *add_type_dialog;
+
 	Vector<Control *> focusables;
 	Timer *update_debounce_timer;
 
 	VBoxContainer *_create_item_list(Theme::DataType p_data_type);
 	void _update_type_list();
 	void _update_type_list_debounced();
-	void _update_add_type_options(const String &p_filter = "");
 	OrderedHashMap<StringName, bool> _get_type_items(String p_type_name, void (Theme::*get_list_func)(StringName, List<StringName> *) const, bool include_default);
 	HBoxContainer *_create_property_control(Theme::DataType p_data_type, String p_item_name, bool p_editable);
 	void _add_focusable(Control *p_control);
 	void _update_type_items();
 
 	void _list_type_selected(int p_index);
-	void _select_type(String p_type_name);
 	void _add_type_button_cbk();
-	void _add_type_filter_cbk(const String &p_value);
-	void _add_type_options_cbk(int p_index);
-	void _add_type_dialog_confirmed();
-	void _add_type_dialog_entered(const String &p_value);
-	void _add_type_dialog_activated(int p_index);
 	void _add_default_type_items();
 
 	void _item_add_cbk(int p_data_type, Control *p_control);
@@ -328,6 +373,11 @@ class ThemeTypeEditor : public MarginContainer {
 	void _unpin_leading_stylebox();
 	void _update_stylebox_from_leading();
 
+	void _type_variation_changed(const String p_value);
+	void _add_type_variation_cbk();
+
+	void _add_type_dialog_selected(const String p_type_name);
+
 protected:
 	void _notification(int p_what);
 	static void _bind_methods();
