@@ -57,6 +57,7 @@ class ScriptCreateDialog : public ConfirmationDialog {
 	OptionButton *language_menu;
 	OptionButton *template_menu;
 	LineEdit *file_path;
+	LineEdit *internal_name;
 	Button *path_button;
 	EditorFileDialog *file_browse;
 	CheckBox *internal;
@@ -81,6 +82,9 @@ class ScriptCreateDialog : public ConfirmationDialog {
 	int default_language;
 	bool re_check_path;
 
+	Control *path_controls[2];
+	Control *name_controls[2];
+
 	enum ScriptOrigin {
 		SCRIPT_ORIGIN_PROJECT,
 		SCRIPT_ORIGIN_EDITOR,
