@@ -54,4 +54,5 @@ public:
 	void set_flag(PhysicsServer::HingeJointFlag p_flag, bool p_value);
 	bool get_flag(PhysicsServer::HingeJointFlag p_flag) const;
 };
-#endif
+
+#endif // HINGE_JOINT_BULLET_H
