@@ -41,34 +41,32 @@
 #define WSS_OUT_BUF PNAME("network/limits/websocket_server/max_out_buffer_kb")
 #define WSS_OUT_PKT PNAME("network/limits/websocket_server/max_out_packets")
 
-/* clang-format off */
-#define GDCICLASS(CNAME) \
-public:\
-	static CNAME *(*_create)();\
-\
-	static Ref<CNAME > create_ref() {\
-\
-		if (!_create)\
-			return Ref<CNAME >();\
-		return Ref<CNAME >(_create());\
-	}\
-\
-	static CNAME *create() {\
-\
-		if (!_create)\
-			return NULL;\
-		return _create();\
-	}\
-protected:\
+#define GDCICLASS(CNAME)              \
+public:                               \
+	static CNAME *(*_create)();       \
+                                      \
+	static Ref<CNAME> create_ref() {  \
+		if (!_create)                 \
+			return Ref<CNAME>();      \
+		return Ref<CNAME>(_create()); \
+	}                                 \
+                                      \
+	static CNAME *create() {          \
+		if (!_create)                 \
+			return NULL;              \
+		return _create();             \
+	}                                 \
+                                      \
+protected:
 
 #define GDCINULL(CNAME) \
-CNAME *(*CNAME::_create)() = NULL;
+	CNAME *(*CNAME::_create)() = NULL;
 
-#define GDCIIMPL(IMPNAME, CNAME) \
-public:\
-	static CNAME *_create() { return memnew(IMPNAME); }\
-	static void make_default() { CNAME::_create = IMPNAME::_create; }\
-protected:\
-/* clang-format on */
+#define GDCIIMPL(IMPNAME, CNAME)                                      \
+public:                                                               \
+	static CNAME *_create() { return memnew(IMPNAME); }               \
+	static void make_default() { CNAME::_create = IMPNAME::_create; } \
+                                                                      \
+protected:
 
 #endif // WEBSOCKET_MACROS_H
