@@ -60,6 +60,10 @@ private:
 	int fixed_fps;
 	bool fractional_delta;
 
+#ifdef TOOLS_ENABLED
+	bool show_visibility_rect;
+#endif
+
 	Ref<Material> process_material;
 
 	DrawOrder draw_order;
@@ -87,6 +91,10 @@ public:
 	void set_process_material(const Ref<Material> &p_material);
 	void set_speed_scale(float p_scale);
 
+#ifdef TOOLS_ENABLED
+	void set_show_visibility_rect(bool p_show_visibility_rect);
+#endif
+
 	bool is_emitting() const;
 	int get_amount() const;
 	float get_lifetime() const;
