@@ -67,7 +67,7 @@ float AnimationNodeAnimation::process(float p_time, bool p_seek) {
 	AnimationPlayer *ap = state->player;
 	ERR_FAIL_COND_V(!ap, 0);
 
-	float time = get_parameter(this->time);
+	const float current_time = get_parameter(this->time);
 
 	if (!ap->has_animation(animation)) {
 		AnimationNodeBlendTree *tree = Object::cast_to<AnimationNodeBlendTree>(parent);
@@ -84,6 +84,7 @@ float AnimationNodeAnimation::process(float p_time, bool p_seek) {
 
 	Ref<Animation> anim = ap->get_animation(animation);
 
+	float time = current_time;
 	float step;
 
 	if (p_seek) {
@@ -103,6 +104,7 @@ float AnimationNodeAnimation::process(float p_time, bool p_seek) {
 
 	} else if (time > anim_size) {
 		time = anim_size;
+		step = anim_size - current_time;
 	}
 
 	blend_animation(animation, time, step, p_seek, 1.0);
@@ -350,7 +352,7 @@ AnimationNodeOneShot::AnimationNodeOneShot() {
 	mix = MIX_MODE_BLEND;
 	sync = false;
 
-	active = "active";
+	active = PNAME("active");
 	prev_active = "prev_active";
 	time = "time";
 	remaining = "remaining";
@@ -397,7 +399,7 @@ void AnimationNodeAdd2::_bind_methods() {
 }
 
 AnimationNodeAdd2::AnimationNodeAdd2() {
-	add_amount = "add_amount";
+	add_amount = PNAME("add_amount");
 	add_input("in");
 	add_input("add");
 	sync = false;
@@ -444,7 +446,7 @@ void AnimationNodeAdd3::_bind_methods() {
 }
 
 AnimationNodeAdd3::AnimationNodeAdd3() {
-	add_amount = "add_amount";
+	add_amount = PNAME("add_amount");
 	add_input("-add");
 	add_input("in");
 	add_input("+add");
@@ -490,7 +492,7 @@ void AnimationNodeBlend2::_bind_methods() {
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "sync"), "set_use_sync", "is_using_sync");
 }
 AnimationNodeBlend2::AnimationNodeBlend2() {
-	blend_amount = "blend_amount";
+	blend_amount = PNAME("blend_amount");
 	add_input("in");
 	add_input("blend");
 	sync = false;
@@ -533,7 +535,7 @@ void AnimationNodeBlend3::_bind_methods() {
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "sync"), "set_use_sync", "is_using_sync");
 }
 AnimationNodeBlend3::AnimationNodeBlend3() {
-	blend_amount = "blend_amount";
+	blend_amount = PNAME("blend_amount");
 	add_input("-blend");
 	add_input("in");
 	add_input("+blend");
@@ -565,7 +567,7 @@ float AnimationNodeTimeScale::process(float p_time, bool p_seek) {
 void AnimationNodeTimeScale::_bind_methods() {
 }
 AnimationNodeTimeScale::AnimationNodeTimeScale() {
-	scale = "scale";
+	scale = PNAME("scale");
 	add_input("in");
 }
 
@@ -601,7 +603,7 @@ void AnimationNodeTimeSeek::_bind_methods() {
 
 AnimationNodeTimeSeek::AnimationNodeTimeSeek() {
 	add_input("in");
-	seek_pos = "seek_position";
+	seek_pos = PNAME("seek_position");
 }
 
 /////////////////////////////////////////////////
@@ -794,7 +796,7 @@ AnimationNodeTransition::AnimationNodeTransition() {
 	prev_xfading = "prev_xfading";
 	prev = "prev";
 	time = "time";
-	current = "current";
+	current = PNAME("current");
 	prev_current = "prev_current";
 	xfade = 0.0;
 
