@@ -29,6 +29,7 @@
 /*************************************************************************/
 
 #include "capsule_shape.h"
+
 #include "servers/physics_server.h"
 
 Vector<Vector3> CapsuleShape::get_debug_mesh_lines() {
@@ -66,6 +67,10 @@ Vector<Vector3> CapsuleShape::get_debug_mesh_lines() {
 	return points;
 }
 
+real_t CapsuleShape::get_enclosing_radius() const {
+	return radius + height * 0.5;
+}
+
 void CapsuleShape::_update_shape() {
 	Dictionary d;
 	d["radius"] = radius;
@@ -102,12 +107,12 @@ void CapsuleShape::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("set_height", "height"), &CapsuleShape::set_height);
 	ClassDB::bind_method(D_METHOD("get_height"), &CapsuleShape::get_height);
 
-	ADD_PROPERTY(PropertyInfo(Variant::REAL, "radius", PROPERTY_HINT_RANGE, "0.01,4096,0.01"), "set_radius", "get_radius");
-	ADD_PROPERTY(PropertyInfo(Variant::REAL, "height", PROPERTY_HINT_RANGE, "0.01,4096,0.01"), "set_height", "get_height");
+	ADD_PROPERTY(PropertyInfo(Variant::REAL, "radius", PROPERTY_HINT_RANGE, "0.001,100,0.001,or_greater"), "set_radius", "get_radius");
+	ADD_PROPERTY(PropertyInfo(Variant::REAL, "height", PROPERTY_HINT_RANGE, "0.001,100,0.001,or_greater"), "set_height", "get_height");
 }
 
 CapsuleShape::CapsuleShape() :
-		Shape(PhysicsServer::get_singleton()->shape_create(PhysicsServer::SHAPE_CAPSULE)) {
+		Shape(RID_PRIME(PhysicsServer::get_singleton()->shape_create(PhysicsServer::SHAPE_CAPSULE))) {
 	radius = 1.0;
 	height = 1.0;
 	_update_shape();
