@@ -46,7 +46,12 @@ public:
 	Plane get_plane() const;
 
 	virtual Vector<Vector3> get_debug_mesh_lines();
+	virtual real_t get_enclosing_radius() const {
+		// Should be infinite?
+		return 0;
+	};
 
 	PlaneShape();
 };
+
 #endif // PLANE_SHAPE_H
