@@ -51,12 +51,16 @@ SceneStringNames::SceneStringNames() {
 	tree_exiting = StaticCString::create("tree_exiting");
 	tree_exited = StaticCString::create("tree_exited");
 	ready = StaticCString::create("ready");
+	child_entered_tree = StaticCString::create("child_entered_tree");
+	child_exiting_tree = StaticCString::create("child_exiting_tree");
 	item_rect_changed = StaticCString::create("item_rect_changed");
 	size_flags_changed = StaticCString::create("size_flags_changed");
 	minimum_size_changed = StaticCString::create("minimum_size_changed");
 	sleeping_state_changed = StaticCString::create("sleeping_state_changed");
 
 	finished = StaticCString::create("finished");
+	loop_finished = StaticCString::create("loop_finished");
+	step_finished = StaticCString::create("step_finished");
 	emission_finished = StaticCString::create("emission_finished");
 	animation_finished = StaticCString::create("animation_finished");
 	animation_changed = StaticCString::create("animation_changed");
