@@ -102,19 +102,24 @@ int PortalRoomsBSP::find_room_within(const PortalRenderer &p_portal_renderer, co
 
 	// first try previous room
 	if (p_previous_room_id != -1) {
-		const VSRoom &prev_room = p_portal_renderer.get_room(p_previous_room_id);
+		if (p_previous_room_id < p_portal_renderer.get_num_rooms()) {
+			const VSRoom &prev_room = p_portal_renderer.get_room(p_previous_room_id);
 
-		// we can only use this shortcut if the room doesn't include internal rooms.
-		// otherwise the point may be inside more than one room, and we need to find the room of highest priority.
-		if (!prev_room._contains_internal_rooms) {
-			closest = prev_room.is_point_within(p_pos);
-			closest_room_id = p_previous_room_id;
+			// we can only use this shortcut if the room doesn't include internal rooms.
+			// otherwise the point may be inside more than one room, and we need to find the room of highest priority.
+			if (!prev_room._contains_internal_rooms) {
+				closest = prev_room.is_point_within(p_pos);
+				closest_room_id = p_previous_room_id;
 
-			if (closest < 0.0) {
-				return p_previous_room_id;
+				if (closest < 0.0) {
+					return p_previous_room_id;
+				}
+			} else {
+				// don't mark it as checked later, as we haven't done it because it contains internal rooms
+				p_previous_room_id = -1;
 			}
 		} else {
-			// don't mark it as checked later, as we haven't done it because it contains internal rooms
+			// previous room was out of range (perhaps due to reconverting room system and the number of rooms decreasing)
 			p_previous_room_id = -1;
 		}
 	}
@@ -526,7 +531,6 @@ int PortalRoomsBSP::evaluate_room_split_plane(int p_room_a_id, int p_room_b_id,
 int PortalRoomsBSP::evaluate_plane(const VSPortal *p_portal, const Plane &p_plane, const LocalVector<int32_t, int32_t> &p_room_ids, LocalVector<int32_t, int32_t> *r_room_ids_back, LocalVector<int32_t, int32_t> *r_room_ids_front) {
 	int rooms_front = 0;
 	int rooms_back = 0;
-	int rooms_split = 0;
 
 	if (r_room_ids_back) {
 		DEV_ASSERT(!r_room_ids_back->size());
@@ -622,8 +626,6 @@ int PortalRoomsBSP::evaluate_plane(const VSPortal *p_portal, const Plane &p_plan
 		if (r_room_ids_back) {
 			r_room_ids_back->push_back(rid);
 		}
-
-		rooms_split++;
 	}
 
 #undef GODOT_BSP_PUSH_BACK
