@@ -28,8 +28,8 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
 /*************************************************************************/
 
-#ifndef SHADERTYPES_H
-#define SHADERTYPES_H
+#ifndef SHADER_TYPES_H
+#define SHADER_TYPES_H
 
 #include "core/ordered_hash_map.h"
 #include "servers/visual_server.h"
@@ -57,4 +57,4 @@ public:
 	ShaderTypes();
 };
 
-#endif // SHADERTYPES_H
+#endif // SHADER_TYPES_H
